import Acquisition
import AccessControl
import OFS
from OFS import content_types

from Globals import HTMLFile, MessageDialog, Persistent
from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String

from AccessControl.Permission import Permission

import string
import time
import sys
import types
import StringIO
import mimetypes
import re
import rfc822
import os
import urllib
import mimify


try:
    import DateTime
except: pass


import Imap
from ImapUtil import CreateStruct, makemap, Internaldate2tuple
import ImapFolder
import ImapMessage

from pprint import pformat

ConnectionError='IMAP Connection Error'

# Imap Root Folder (the zope Folder object)

class ImapMailbox(ImapFolder.ImapFolder,
    AccessControl.Role.RoleManager):
	'''
	ImapMailbox Product
	'''

	name='ImapMailbox'
	id='ImapMailbox'
	icon='misc_/ImapClient/Mailbox'
	meta_type='ImapMailbox'
	
	title='Mailbox'
	
	isAImapMailbox = 1
	
        _properties = (
                {'id':'title','type':'string','mode':'w'},
                {'id':'_username','type':'string','mode':'w'},
		{'id':'_password','type':'string','mode':'w'},
                {'id':'_host','type':'string','mode':'w'},
                {'id':'_Folder','type':'string','mode':'w'},		
                )


	__ac_permissions__=(
		('View management screens',('manage_tabs','manage_main')),
		('Change permissions',('manage_access',)),
		('Change ImapMailbox',('manage_edit',),('Manager',)),
		)
		

	def __init__(self,id,title='Mailbox',username='',password='',host='localhost',userfolder='INBOX.'):

		self._v_conn = None
		self._folder_uptodate = 0
		self._v_last_selected = None


		self.id=id
		self.title=title
		self._username=username
		self._password=password
		self._host=host
		
		if userfolder in ['INBOX','INBOX.']:
			self.isAMessageFolder = 1
		else:
			self.isAMessageFolder = 0
		
		ImapFolder.ImapFolder.__init__.im_func(self,id,userfolder)




	## Zope Management


	def manage_edit(self,title='Mailbox',username='',password='',host='localhost',userfolder='INBOX.',REQUEST=None):
		"""edit ImapMailbox"""
		self.title=title
		self._username=username
		self._password=password
		self._host=host
		self._Folder = userfolder

		if REQUEST is not None:
			return MessageDialog(
				title='Edited',
				message='<strong>%s</strong> has been edited.' % self.id,
				action = './manage_main'
				)

	## ObjectManager Patches


	def canBrowse(self):
		return self.isOpen()


	## Connection Handling

	def _connection(self):
       	    if not self.isOpen(): self._open()
            return self._v_conn


	def isOpen(self):
	    " quickly checks to see if the connection's open "
	    if not hasattr(self, '_v_conn'):
        	self._v_conn = None
	    if self._v_conn is None:
        	return 0
	    elif self._v_conn.noop()[0] != 'OK':
		 self._v_conn = None
		 return 0
	    else:
        	 return 1

	def _open(self):
	    """ open a connection """
	    try:
        	self._close()
	    except:
        	pass
	    self._v_conn = self.getNewConnection()


	def _close(self):
		M = self._v_conn
		M.close()
		M.logout()
		self._v_conn = None
		
		
	def getNewConnection(self):
	        try:
			conn=Imap.IMAP4(self._host)
			if conn.login(self._username, self._password):
                        	return conn
                        else:
                        	return None
	        except:
        		return None

	def getImapConnection(self, Folder=None, rw=0):
	       	if not Folder:
			Folder=self._Folder
		
        	M=self._v_conn=self._connection()
		
		if not M:
			raise "Connection Error"
		
		if Folder:
			resp = []
			if Folder[-1] == '.':
				Folder = Folder[:-1]

			if not hasattr(self, '_v_last_selected'):
        			self._v_last_selected = None

			if self._v_last_selected == (Folder,rw):
				return M
			else:
				self._v_last_selected = (Folder,rw)
				if rw: resp = M.select(Folder)
                		else:  resp = M.select(Folder,1)

                return M




	## debug stuff
	


	def test_html(self,REQUEST=None):
		''' official '''
		x = self.objectValues()
		ret = "%s :" % x
		for y in x:
			ret = ret + "1.(%s)" % y.PaddedName + "\n<br>"
			for z in y.GetChildren():
				ret = ret + "2.(%s)" % z.PaddedName + "\n<br>"
				for a in z.GetChildren():
					ret = ret + "3.(%s)" % a.PaddedName + "\n<br>"
					
		ret = ret + "<hr><br>"
		
		for x,y in self.__dict__.items():
			ret = ret + "%s: %s\n<br>" % (x,y)
			
		return "<html><body>"+ret+"</body></html>"
		
	def request_html(self,REQUEST=None):
		''' official '''
		ret = self.id + "<br>\n"
		if REQUEST:
			ret = "%s<br>\n" % REQUEST
		else:
			ret = "empty<br>\n"
		return ret
			
			
	def checkaq(self,REQUEST=None):
		''' test '''
		ret = []
		a = ret.append
		ob = self.aq_inner
		while hasattr(ob,'aq_parent'):
			a(str(ob.id))
			ob = getattr(ob,'aq_parent')

		return "<html><body>" + string.join(ret,'<br>\n') + "</html></body>"
			


import Globals
for klass in (ImapMailbox,):
    Globals.default__class_init__(klass)







